{$I DacDemo.inc}

unit Query;

interface

uses
  OdacDemoForm,
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
{$IFNDEF KYLIX}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DBCtrls, ExtCtrls,
  Grids, DBGrids, StdCtrls, ToolWin, ComCtrls, Buttons, OdacVcl,
{$ELSE}
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs, QStdCtrls,
  QDBCtrls, QComCtrls, QExtCtrls, QGrids, QDBGrids, QButtons, OdacClx,
{$ENDIF}
{$IFDEF CLR}
  System.ComponentModel,
{$ENDIF}
{$IFDEF FPC}
  LResources,
  MemDataSet,
{$ELSE}
  MemDS,
{$ENDIF}
  DBAccess, Ora, DB, OdacDemoFrame;

type
  TQueryFrame = class(TOdacDemoFrame)
    OraQuery: TOraQuery;
    DBGrid: TDBGrid;
    DataSource: TDataSource;
    meSQL: TMemo;
    Panel4: TPanel;
    ToolBar: TPanel;
    btClose: TSpeedButton;
    btPrepare: TSpeedButton;
    btUnPrepare: TSpeedButton;
    btExecute: TSpeedButton;
    btBreak: TSpeedButton;
    btOpen: TSpeedButton;
    Panel3: TPanel;
    cbFetchAll: TCheckBox;
    cbNonBlocking: TCheckBox;
    Panel2: TPanel;
    Panel1: TPanel;
    btLock: TSpeedButton;
    btRefreshRecord: TSpeedButton;
    btShowState: TSpeedButton;
    DBNavigator1: TDBNavigator;
    Splitter1: TSplitter;
    btSaveToXML: TSpeedButton;
    SaveDialog: TSaveDialog;
    procedure btOpenClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btPrepareClick(Sender: TObject);
    procedure btUnPrepareClick(Sender: TObject);
    procedure btExecuteClick(Sender: TObject);
    procedure meSQLExit(Sender: TObject);
    procedure btBreakClick(Sender: TObject);
    procedure cbNonBlockingClick(Sender: TObject);
    procedure OraQueryAfterExecute(Sender: TObject; Result: Boolean);
    procedure cbFetchAllClick(Sender: TObject);
    procedure btLockClick(Sender: TObject);
    procedure btRefreshRecordClick(Sender: TObject);
    procedure btShowStateClick(Sender: TObject);
    procedure btSaveToXMLClick(Sender: TObject);
  private
    { Private declarations }
    procedure ShowState;
  public
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

procedure TQueryFrame.ShowState;
var
  St:string;

  procedure AddSt(S:string);
  begin
    if St <> '' then
      St := St + ', ';
    St := St + S;
  end;

begin
  St := '';

  if OraQuery.Prepared then begin
    AddSt('Prepared');

    if OraQuery.IsQuery then
      AddSt('IsQuery');

    if OraQuery.IsPLSQL then
      AddSt('IsPLSQL');
  end;

  if OraQuery.Active then
    AddSt('Active')
  else
    AddSt('Inactive');

  if OraQuery.Executing then
    AddSt('Executing');

  if OraQuery.Fetching then
    AddSt('Fetching');

  OdacForm.StatusBar.Panels[1].Text := St;
end;

procedure TQueryFrame.meSQLExit(Sender: TObject);
begin
  if Trim(OraQuery.SQL.Text) <> Trim(meSQL.Lines.Text) then
    OraQuery.SQL.Text := meSQL.Lines.Text;
  ShowState;
end;

procedure TQueryFrame.btOpenClick(Sender: TObject);
begin
  try
    OraQuery.SQL.Text := meSQL.Lines.Text;
    OraQuery.Open;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btCloseClick(Sender: TObject);
begin
  OraQuery.Close;
  ShowState;
end;

procedure TQueryFrame.btPrepareClick(Sender: TObject);
begin
  try
    OraQuery.SQL.Text := meSQL.Lines.Text;
    OraQuery.Prepare;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btUnPrepareClick(Sender: TObject);
begin
  OraQuery.UnPrepare;
  ShowState;
end;

procedure TQueryFrame.btExecuteClick(Sender: TObject);
begin
  try
    OraQuery.SQL.Text := meSQL.Lines.Text;
    OraQuery.Execute;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.btBreakClick(Sender: TObject);
begin
  try
    OraQuery.BreakExec;
  finally
    ShowState;
  end;
end;

procedure TQueryFrame.cbNonBlockingClick(Sender: TObject);
begin
  OraQuery.NonBlocking := cbNonBlocking.Checked;
end;

procedure TQueryFrame.OraQueryAfterExecute(Sender: TObject; Result: Boolean);
begin
  ShowState;

  if Result then
    ODACForm.StatusBar.Panels[1].Text := ODACForm.StatusBar.Panels[1].Text + '   >>>> Success'
  else
    ODACForm.StatusBar.Panels[1].Text := ODACForm.StatusBar.Panels[1].Text + '   >>>> Fail';

{$IFNDEF LINUX}
  MessageBeep(1);
{$ENDIF}
end;

procedure TQueryFrame.cbFetchAllClick(Sender: TObject);
begin
  OraQuery.FetchAll := cbFetchAll.Checked;
end;

{SELECT * FROM Scott.Dept}

procedure TQueryFrame.btLockClick(Sender: TObject);
begin
  OraQuery.Lock;
end;

procedure TQueryFrame.btRefreshRecordClick(Sender: TObject);
begin
  OraQuery.RefreshRecord;
end;

procedure TQueryFrame.btShowStateClick(Sender: TObject);
begin
  ShowState;
end;

procedure TQueryFrame.btSaveToXMLClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    OraQuery.SaveToXML(SaveDialog.FileName);
end;

//
procedure TQueryFrame.Initialize;
begin
  inherited;

  OraQuery.Connection := Connection;

  meSQL.Lines.Assign(OraQuery.SQL);
  ShowState;

  cbNonBlocking.Checked := OraQuery.NonBlocking;
  cbFetchAll.Checked := OraQuery.FetchAll;
end;

procedure TQueryFrame.SetDebug(Value: boolean);
begin
  OraQuery.Debug := Value;
end;

initialization
{$IFDEF FPC}
{$I Query.lrs}
{$ENDIF}

end.
